@@ -1,6 +1,6 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) CERN 2013  
+ * Copyright (C) CERN 2013
  * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
@@ -19,6 +19,7 @@
  *
 */
 
+#include <mutex>
 #include <davix_internal.hpp>
 #include "davix_reliability_ops.hpp"
 
@@ -32,14 +33,13 @@ namespace Davix{
 
 
 using namespace StrUtil;
-using namespace boost;
 
-typedef function< dav_ssize_t (IOChainContext &)> FuncIO;
-typedef function< StatInfo & (IOChainContext &) > FuncStatInfo;
+typedef std::function< dav_ssize_t (IOChainContext &)> FuncIO;
+typedef std::function< StatInfo & (IOChainContext &) > FuncStatInfo;
 
 
 static bool metalink_support_disabled=false;
-static once_flag metalink_once = BOOST_ONCE_INIT;
+static std::once_flag metalink_once;
 
 
 void propagateNonRecoverableExceptions(DavixException & e){
@@ -55,7 +55,7 @@ void metalink_check(){
 }
 
 static bool isMetalinkDisabled(const RequestParams* params){
-    call_once(metalink_check, metalink_once);
+    std::call_once(metalink_once, metalink_check);
     return (params != NULL && params->getMetalinkMode() == MetalinkMode::Disable) || metalink_support_disabled;
 }
 
@@ -151,10 +151,10 @@ int davix_get_metalink_url( Context & c, const Uri & uri,
 
 
     DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Executing head query to {} for Metalink file", uri.getString());
-    
-    
+
+
     if(tmp_err != NULL || (req.executeRequest(&tmp_err) <0)) {
-      if (tmp_err) 
+      if (tmp_err)
         throw DavixException(davix_scope_meta(), tmp_err->getStatus(), tmp_err->getErrMsg());
       else
         throw DavixException(davix_scope_meta(), Davix::StatusCode::UnknowError, "Unknown error");
@@ -239,18 +239,18 @@ std::vector<File> & MetalinkOps::getReplicas(IOChainContext & iocontext, std::ve
 }
 
 StatInfo & MetalinkOps::statInfo(IOChainContext &iocontext, StatInfo &st_info){
-    FuncStatInfo func(bind(&HttpIOChain::statInfo, _next.get(), _1, ref(st_info)));
+    FuncStatInfo func(std::bind(&HttpIOChain::statInfo, _next.get(), std::placeholders::_1, std::ref(st_info)));
     return metalinkExecutor<FuncStatInfo, StatInfo &>(*this, iocontext, func);
 }
 
 dav_ssize_t MetalinkOps::read(IOChainContext &iocontext, void *buf, dav_size_t count){
-    FuncIO func(bind(&HttpIOChain::read, _next.get(),_1, buf, count));
+    FuncIO func(std::bind(&HttpIOChain::read, _next.get(), std::placeholders::_1, buf, count));
     return metalinkExecutor<FuncIO, dav_ssize_t>(*this, iocontext, func);
 }
 
 dav_ssize_t MetalinkOps::pread(IOChainContext &iocontext, void *buf, dav_size_t count, dav_off_t offset){
 
-    FuncIO func(bind(&HttpIOChain::pread, _next.get(),_1, buf, count, offset));
+    FuncIO func(std::bind(&HttpIOChain::pread, _next.get(), std::placeholders::_1, buf, count, offset));
     return metalinkExecutor<FuncIO, dav_ssize_t>(*this, iocontext, func);
 }
 
@@ -259,13 +259,13 @@ dav_ssize_t MetalinkOps::preadVec(IOChainContext & iocontext, const DavIOVecInpu
                           DavIOVecOuput * output_vec,
                           const dav_size_t count_vec){
 
-    FuncIO func(bind(&HttpIOChain::preadVec, _next.get(),_1, input_vec, output_vec, count_vec));
+    FuncIO func(std::bind(&HttpIOChain::preadVec, _next.get(), std::placeholders::_1, input_vec, output_vec, count_vec));
     return metalinkExecutor<FuncIO, dav_ssize_t>(*this, iocontext, func);
 }
 
 // read to fd Metalink manager
 dav_ssize_t MetalinkOps::readToFd(IOChainContext & iocontext, int fd, dav_size_t size){
-    FuncIO func(bind(&HttpIOChain::readToFd, _next.get(), _1, fd, size));
+    FuncIO func(std::bind(&HttpIOChain::readToFd, _next.get(), std::placeholders::_1, fd, size));
     return metalinkExecutor<FuncIO, dav_ssize_t>(*this, iocontext, func);
 }
 
@@ -334,18 +334,18 @@ AutoRetryOps::~AutoRetryOps(){
 
 
 StatInfo & AutoRetryOps::statInfo(IOChainContext &iocontext, StatInfo &st_info){
-    FuncStatInfo func(bind(&HttpIOChain::statInfo, _next.get(), _1, ref(st_info)));
+    FuncStatInfo func(std::bind(&HttpIOChain::statInfo, _next.get(), std::placeholders::_1, std::ref(st_info)));
     return autoRetryExecutor<FuncStatInfo, StatInfo &>(*this, iocontext, func);
 }
 
 dav_ssize_t AutoRetryOps::read(IOChainContext &iocontext, void *buf, dav_size_t count){
-    FuncIO func(bind(&HttpIOChain::read, _next.get(),_1, buf, count));
+    FuncIO func(std::bind(&HttpIOChain::read, _next.get(), std::placeholders::_1, buf, count));
     return autoRetryExecutor<FuncIO, dav_ssize_t>(*this, iocontext, func);
 }
 
 dav_ssize_t AutoRetryOps::pread(IOChainContext &iocontext, void *buf, dav_size_t count, dav_off_t offset){
 
-    FuncIO func(bind(&HttpIOChain::pread, _next.get(),_1, buf, count, offset));
+    FuncIO func(std::bind(&HttpIOChain::pread, _next.get(), std::placeholders::_1, buf, count, offset));
     return autoRetryExecutor<FuncIO, dav_ssize_t>(*this, iocontext, func);
 }
 
@@ -354,16 +354,15 @@ dav_ssize_t AutoRetryOps::preadVec(IOChainContext & iocontext, const DavIOVecInp
                           DavIOVecOuput * output_vec,
                           const dav_size_t count_vec){
 
-    FuncIO func(bind(&HttpIOChain::preadVec, _next.get(),_1, input_vec, output_vec, count_vec));
+    FuncIO func(std::bind(&HttpIOChain::preadVec, _next.get(), std::placeholders::_1, input_vec, output_vec, count_vec));
     return autoRetryExecutor<FuncIO, dav_ssize_t>(*this, iocontext, func);
 }
 
 // read to fd Metalink manager
 dav_ssize_t AutoRetryOps::readToFd(IOChainContext & iocontext, int fd, dav_size_t size){
-    FuncIO func(bind(&HttpIOChain::readToFd, _next.get(), _1, fd, size));
+    FuncIO func(std::bind(&HttpIOChain::readToFd, _next.get(), std::placeholders::_1, fd, size));
     return autoRetryExecutor<FuncIO, dav_ssize_t>(*this, iocontext, func);
 }
 
 
 } // namespace Davix
-
