@@ -5,12 +5,7 @@
 #include <map>
 #include <limits>
 #include <utility>
-
-#ifndef NO_BOOST_INCLUDE
-
-#include <boost/shared_ptr.hpp>
-#include <boost/thread/mutex.hpp>
-#include <boost/thread/locks.hpp>
+#include <mutex>
 
 
 namespace A_LIB_NAMESPACE{
@@ -20,8 +15,8 @@ namespace A_LIB_NAMESPACE{
 ///
 template <class Key, class Value, class CompareT = std::less<Key> >
 class Cache {
-    typedef std::map<Key, boost::shared_ptr<Value> > Map;
-    typedef boost::shared_ptr<Value> shrPtr_type;
+    typedef std::map<Key, std::shared_ptr<Value> > Map;
+    typedef std::shared_ptr<Value> shrPtr_type;
 
 public:
     Cache(size_t max_size = std::numeric_limits<size_t>::max()) :
@@ -40,7 +35,7 @@ public:
     /// \return
     ///
     shrPtr_type insert(const Key & key, const shrPtr_type & value){
-        boost::lock_guard<boost::mutex> l(_m);
+        std::lock_guard<std::mutex> l(_m);
         auto_clean();
         std::pair< typename Map::iterator, bool > stored = map.insert(std::pair<Key,shrPtr_type> (key, value) );
         if(stored.second)
@@ -55,7 +50,7 @@ public:
     /// \return
     ///
     shrPtr_type find(const Key & key){
-        boost::lock_guard<boost::mutex> l(_m);
+        std::lock_guard<std::mutex> l(_m);
         typename Map::iterator it = map.find( key);
         if(it == map.end())
             return shrPtr_type();
@@ -68,7 +63,7 @@ public:
     /// \return the next key
     ///
     const Key upper_bound(const Key & key) {
-        boost::lock_guard<boost::mutex> l(_m);
+        std::lock_guard<std::mutex> l(_m);
         typename Map::iterator it = map.upper_bound( key);
         if(it == map.end())
             return Key();
@@ -80,7 +75,7 @@ public:
     /// \return
     ///
     size_t getSize() const{
-        boost::lock_guard<boost::mutex> l(_m);
+        std::lock_guard<std::mutex> l(_m);
         return map.size();
     }
 
@@ -90,7 +85,7 @@ public:
     /// \return
     ///
     shrPtr_type take( const Key & key){
-        boost::lock_guard<boost::mutex> l(_m);
+        std::lock_guard<std::mutex> l(_m);
         typename Map::iterator it = map.find( key);
         if(it == map.end())
             return shrPtr_type();
@@ -104,12 +99,12 @@ public:
     /// \return
     ///
     bool erase( const Key & key){
-        boost::lock_guard<boost::mutex> l(_m);
+        std::lock_guard<std::mutex> l(_m);
         return ( map.erase(key) != 0);
     }
 
     void clear(){
-        boost::lock_guard<boost::mutex> l(_m);
+        std::lock_guard<std::mutex> l(_m);
         map.clear();
     }
 
@@ -118,7 +113,7 @@ protected:
     CompareT cmp;
     Map map;
     size_t _max_size;
-    mutable boost::mutex _m;
+    mutable std::mutex _m;
 
     void auto_clean(){
         if(map.size() == _max_size){
@@ -131,7 +126,3 @@ protected:
 
 
 }
-
-
-#endif
-
