@@ -29,7 +29,7 @@
 #include <davix_context_internal.hpp>
 #include <neon/neonsession.hpp>
 #include <fileops/fileutils.hpp>
-
+#include <fileops/AzureIO.hpp>
 
 
 
@@ -166,7 +166,10 @@ NEONRequest::NEONRequest(HttpRequest & h, Context& context, const Uri & uri_req)
     req_started(false),
     req_running(false),
     _last_request_flag(0),
-    _headers_field(){
+    _headers_field(),
+    _accepted_202_retries(0),
+    _early_termination(0),
+    _early_termination_error(NULL) {
 }
 
 
@@ -199,7 +202,10 @@ int NEONRequest::createRequest(DavixError** err){
        resetRequest();
     }
 
-    boost::shared_ptr<Uri> redir_url = ContextExplorer::SessionFactoryFromContext(_c).redirectionResolve(_request_type, *_current);
+    std::shared_ptr<Uri> redir_url;
+    if(this->params.getTransparentRedirectionSupport()) {
+        redir_url = ContextExplorer::SessionFactoryFromContext(_c).redirectionResolve(_request_type, *_current);
+    }
 
     // performing an operation which could change the PFN? Clear all cache entries for selected URL
     if(_request_type == "DELETE" || _request_type == "MOVE") {
@@ -257,6 +263,10 @@ bool NEONRequest::checkTimeout(DavixError **err){
     return false;
 }
 
+static dav_ssize_t iocontext_content_provider(HttpBodyProvider provider, void* udata, void* buffer, dav_size_t size) {
+    return provider(udata, (char*) buffer, size);
+}
+
 void NEONRequest::configureRequest(){
 
     // add custom user headers, but make sure they're only added once
@@ -265,13 +275,37 @@ void NEONRequest::configureRequest(){
         std::copy(params.getHeaders().begin(), params.getHeaders().end(), std::back_inserter(_headers_field));
     }
 
-    /* Doing a PUT to Azure? Add the necessary Microsoft-specific header */
+    /* Doing a PUT to Azure? Horrific workaround to enforce the odd two-step
+       custom Azure procedure, even when receiving a redirect. */
     if(_request_type == "PUT" &&
-       _current.get()->queryParamExists("sig") &&
-       _current.get()->queryParamExists("sr") &&
-       _current.get()->queryParamExists("sp")) {
+         _current.get()->queryParamExists("sig") &&
+         _current.get()->queryParamExists("sr") &&
+         _current.get()->queryParamExists("sp") && !_current.get()->fragmentParamExists("azuremechanism")) {
 
-        _headers_field.push_back(HeaderLine("x-ms-blob-type", "BlockBlob"));
+        IOChainContext iocontext(_c, *_current, &params);
+
+        using std::placeholders::_1;
+        using std::placeholders::_2;
+
+        try {
+          AzureIO azureio;
+
+          if(_fd_content > 0) {
+              azureio.writeFromFd(iocontext, _fd_content, _content_len);
+          }
+          else if(_content_provider.callback) {
+              DataProviderFun provider = std::bind(iocontext_content_provider, _content_provider.callback, _content_provider.udata, _1, _2);
+              azureio.writeFromCb(iocontext, provider, _content_len);
+          }
+          else if(_content_ptr && _content_len > 0) {
+              throw DavixException("neon request", StatusCode::InvalidArgument, "unable to use Azure PUT by providing a direct string");
+          }
+        }
+        catch(DavixException &e) {
+            e.toDavixError(&_early_termination_error);
+        }
+
+        _early_termination = true;
     }
 
     // setup timeout
@@ -306,14 +340,22 @@ void NEONRequest::configureRequest(){
 
 
 void NEONRequest::configureS3params(){
-    Uri signed_url = S3::signURI(params, _request_type, *_current, _headers_field, NEON_S3_SIGN_DURATION);
-    _current= boost::shared_ptr<Uri>(new Uri(signed_url));
+    // strange workaround to get S3 compatibility on gcloud to work
+    if(params.getAwsRegion().empty()) {
+        HeaderVec vec = _headers_field;
+        S3::signRequest(params, _request_type, *_current, vec);
+        vec.swap(_headers_field);
+    }
+    else {
+        Uri signed_url = S3::signURI(params, _request_type, *_current, _headers_field, NEON_S3_SIGN_DURATION);
+        _current= std::shared_ptr<Uri>(new Uri(signed_url));
+    }
 }
 
 // TODO: make static?
 void NEONRequest::configureAzureParams(){
     Uri signed_url = Azure::signURI(params.getAzureKey(), _request_type, *_current, NEON_S3_SIGN_DURATION);
-    _current= boost::shared_ptr<Uri>(new Uri(signed_url));
+    _current= std::shared_ptr<Uri>(new Uri(signed_url));
 }
 
 int NEONRequest::processRedirection(int neonCode, DavixError **err){
@@ -403,6 +445,21 @@ int NEONRequest::negotiateRequest(DavixError** err){
         _last_read =1;
         code = getRequestCode();
         switch(code){
+            case 202:
+                // azure will reply with 202 when deleting files :(
+                // Only activate retries on GET.
+                if(_request_type != "GET") goto default_label;
+
+                _accepted_202_retries++;
+                if(_accepted_202_retries <= params.getAcceptedRetry()) {
+                  std::cerr << "DAVIX: Received 202-Accepted, sleeping for " <<
+                    params.getAcceptedRetryDelay() << " seconds before retrying. (attempt " <<
+                    _accepted_202_retries << " out of " << params.getAcceptedRetry() << ")" << std::endl;
+                  sleep(params.getAcceptedRetryDelay());
+                  requestCleanup();
+                  return startRequest(err);
+                }
+                goto default_label;
             case 301:
             case 302:
             case 303:
@@ -410,6 +467,12 @@ int NEONRequest::negotiateRequest(DavixError** err){
                 if( (end_status = processRedirection(status, err)) <0){
                    return -1;
                 }
+
+                if(_early_termination) {
+                    end_status = 0;
+                    break;
+                }
+
                 _number_try--;
                 _redirects++;
 
@@ -457,6 +520,7 @@ int NEONRequest::negotiateRequest(DavixError** err){
                 end_status = 0;
                 break;
             default:
+            default_label:
                 if(code >= 400) {
                     httpcodeToDavixError(code, davix_scope_http_request(), "", err);
                 }
@@ -502,7 +566,7 @@ bool NEONRequest::requestCleanup(){
 }
 
 int NEONRequest::redirectRequest(DavixError **err){
-    boost::shared_ptr<Uri> old_uri;
+    std::shared_ptr<Uri> old_uri;
     const ne_uri * new_uri = ne_redirect_location(_neon_sess->get_ne_sess());
     if(!new_uri){
         DavixError::setupError(err, davix_scope_http_request(), StatusCode::UriParsingError, "Impossible to get the new redirected destination");
@@ -514,7 +578,7 @@ int NEONRequest::redirectRequest(DavixError **err){
 
     // setup new path & session target
     old_uri = _current;
-    _current= boost::shared_ptr<Uri>(new Uri(dst_uri));
+    _current= std::shared_ptr<Uri>(new Uri(dst_uri));
     ne_free(dst_uri);
     ContextExplorer::SessionFactoryFromContext(_c).addRedirection(_request_type, *old_uri, _current);
 
@@ -780,6 +844,10 @@ void NEONRequest::clearAnswerContent(){
 
 
 int NEONRequest::getRequestCode(){
+    if(_early_termination) {
+        if(!_early_termination_error) return 200;
+        return _early_termination_error->getStatus();
+    }
     return ne_get_status(_req)->code;
 }
 
@@ -892,6 +960,7 @@ void NEONRequest::setRequestBody(HttpBodyProvider provider, dav_size_t len, void
 
 
 void NEONRequest::freeRequest(){
+    DavixError::clearError(&_early_termination_error);
     if(_req != NULL){
         ne_request_destroy(_req);
         _req=NULL;
