@@ -51,14 +51,14 @@ struct X509Data{
 
     X509Credential _cred;
 
-    static X509Data* instance(boost::shared_ptr<X509Data> & cred_ptr){
+    static X509Data* instance(std::shared_ptr<X509Data> & cred_ptr){
         if(cred_ptr.get() == NULL){
             cred_ptr.reset(new X509Data());
         }
         return cred_ptr.get();
     }
 
-    static X509Data* reset(boost::shared_ptr<X509Data> & cred_ptr){
+    static X509Data* reset(std::shared_ptr<X509Data> & cred_ptr){
         cred_ptr.reset(new X509Data());
         return cred_ptr.get();
     }
@@ -110,7 +110,9 @@ struct RequestParamsInternal{
         retry_number(default_retry_number),
         retry_delay(),
         _copy_mode(CopyMode::Push),
-        _support_100continue(true)
+        _support_100continue(true),
+        _accepted_retry(180), // wait for half an hour by default
+        _accepted_delay(10)
     {
         timespec_clear(&connexion_timeout);
         timespec_clear(&ops_timeout);
@@ -155,7 +157,9 @@ struct RequestParamsInternal{
         retry_number(param_private.retry_number),
         retry_delay(param_private.retry_delay),
         _copy_mode(param_private._copy_mode),
-        _support_100continue(param_private._support_100continue) {
+        _support_100continue(param_private._support_100continue),
+        _accepted_retry(param_private._accepted_retry),
+        _accepted_delay(param_private._accepted_delay) {
 
         timespec_copy(&(connexion_timeout), &(param_private.connexion_timeout));
         timespec_copy(&(ops_timeout), &(param_private.ops_timeout));
@@ -174,7 +178,7 @@ struct RequestParamsInternal{
     std::vector<std::string> _ca_path;
 
     // auth info
-    boost::shared_ptr<X509Data> _x509_data;
+    std::shared_ptr<X509Data> _x509_data;
 
     std::pair<std::string,std::string> _idlogpass;
     authCallbackLoginPasswordBasic _call_loginpswwd;
@@ -200,7 +204,7 @@ struct RequestParamsInternal{
     HeaderVec _customhdr;
 
     // Proxy server URI
-    boost::shared_ptr<Uri> _proxy_server;
+    std::shared_ptr<Uri> _proxy_server;
 
     // session flag
     int _session_flag;
@@ -223,6 +227,12 @@ struct RequestParamsInternal{
     // whether server has support for 100-Continue
     bool _support_100continue;
 
+    // number of retries in case davix receives 202-Accepted
+    int _accepted_retry;
+
+    // delay in seconds between retries in case davix receives 202-Accepted
+    int _accepted_delay;
+
     // method
     inline void regenerateStateUid(){
         _state_uid = get_requeste_uid();
@@ -280,7 +290,7 @@ void RequestParams::setClientCertX509(const X509Credential & cli_cert){
     d_ptr->regenerateStateUid();
     X509Data* x509 = X509Data::reset(d_ptr->_x509_data);
     x509->_cred = cli_cert;
-    x509->_x509_fun = bind(&X509Data::cred_callback, x509, _1, _2);
+    x509->_x509_fun = std::bind(&X509Data::cred_callback, x509, std::placeholders::_1, std::placeholders::_2);
 }
 
 void RequestParams::setClientLoginPassword(const std::string & login, const std::string & password){
@@ -304,7 +314,7 @@ void RequestParams::setClientCertCallbackX509(authCallbackClientCertX509 callbac
     d_ptr->regenerateStateUid();
     X509Data* x509 = X509Data::reset(d_ptr->_x509_data);
     x509->_pair = std::make_pair(callback, userdata);
-    x509->_x509_fun = bind(&X509Data::c_callback, x509, _1, _2);
+    x509->_x509_fun = std::bind(&X509Data::c_callback, x509, std::placeholders::_1, std::placeholders::_2);
 }
 
 
@@ -539,6 +549,22 @@ bool RequestParams::get100ContinueSupport() const {
   return d_ptr->_support_100continue;
 }
 
+int RequestParams::getAcceptedRetry() const {
+  return d_ptr->_accepted_retry;
+}
+
+void RequestParams::setAcceptedRetry(int retries) {
+  d_ptr->_accepted_retry = retries;
+}
+
+int RequestParams::getAcceptedRetryDelay() const {
+  return d_ptr->_accepted_delay;
+}
+
+void RequestParams::setAcceptedRetryDelay(int delay) {
+  d_ptr->_accepted_delay = delay;
+}
+
 // suppress useless warning
 #pragma GCC diagnostic ignored "-Wint-to-pointer-cast"
 void* RequestParams::getParmState() const{
