@@ -1,6 +1,6 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) CERN 2013  
+ * Copyright (C) CERN 2013
  * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
@@ -21,7 +21,6 @@
 
 #include <davix_internal.hpp>
 #include <utils/davix_uri.hpp>
-#include <boost/algorithm/string/replace.hpp>
 
 
 namespace Davix {
@@ -340,9 +339,40 @@ std::string Uri::unescapeString(const std::string & str){
 
 std::string Uri::queryParamEscape(const std::string & str) {
     std::string tmp = escapeString(str);
-    return boost::replace_all_copy(tmp, "/", "%2F");
+
+    // replace all '/' with '%2F'
+    std::stringstream ss;
+    for(size_t i = 0; i < tmp.size(); i++) {
+        if(tmp[i] == '/') {
+            ss << "%2F";
+        }
+        else {
+            ss << tmp[i];
+        }
+    }
+
+    return ss.str();
+
 }
 
+// Determine whether this is a URL, and if so, URI-escape the right part.
+// Otherwise, simple string concatenation.
+std::string Uri::join(const std::string &left, const std::string &right) {
+    Uri leftUri(left);
+
+    if(leftUri.getStatus() != StatusCode::OK) {
+        if(left.size() == 0 || left[left.size()-1] != '/') {
+            return left + "/" + right;
+        }
+        return left + right;
+    }
+
+    leftUri.ensureTrailingSlash();
+    return leftUri.getString() + Uri::escapeString(right);
+}
+
+
+
 // FIX IT : does not manage properly ../
 Uri Uri::fromRelativePath(const Uri &uri, const std::string &relPath){
     std::ostringstream ss;
@@ -712,5 +742,3 @@ std::ostream& operator<< (std::ostream& stream, const Davix::Uri & _u){
 	stream << _u.getString();
 	return stream;
 }
-
-
